@@ -101,6 +101,15 @@ data Settings = Settings
       -- ^ A log function. Default: no action.
       --
       -- Since 3.X.X.
+    , settingsServerPushLogger :: Request -> ByteString -> Integer -> IO ()
+      -- ^ A HTTP/2 server push log function. Default: no action.
+      --
+      -- Since 3.X.X.
+    , settingsGracefulShutdownTimeout :: Maybe Int
+      -- ^ An optional timeout to limit the time (in seconds) waiting for
+      -- a graceful shutdown of the web server.
+      --
+      -- Since 3.2.8
     }
 
 -- | Specify usage of the PROXY protocol.
@@ -135,6 +144,8 @@ defaultSettings = Settings
     , settingsSlowlorisSize = 2048
     , settingsHTTP2Enabled = True
     , settingsLogger = \_ _ _ -> return ()
+    , settingsServerPushLogger = \_ _ _ -> return ()
+    , settingsGracefulShutdownTimeout = Nothing
     }
 
 -- | Apply the logic provided by 'defaultOnException' to determine if an
